%Periods indicates the number of periods in the irf, but has to be changed
%by hand in the mod file as well.

%This file generates the results of the paper "Bank Leverage Cycles and the External Finance
%Premium" by Ansgar Rannenberg. Executing it with the default 
%options generates figures 1-7, 
%and the second moments reported in Tables 4-6. The IRFs plotted in Figures
%1,2, 5 and 6 are stored in temp_Full_IRF.mat, in temp_BGG_IRF.mat, 
%temp_GK_IRF.mat and temp_nofr_IRF.mat for the full model, the BGG model, 
%and the GK model, respectively. The simulation results reported in
%Figure 7 are stored in temp_Crisis.mat. 
%The second moments are stored in temp_Mo_Full.mat, temp_Mo_BGG.mat and 
%temp_Mo_GK.mat, and are listed in the sequence they appear in the table.
%Additionally, the final three entries of temp_Mo_GK.mat list the second 
%moments of loans, the bank capital ratio and bank net worth defined
%according to the "Fair value method" suggested by Gertler and Karadi
%(2013). 

%Robustness checks:
%To generate those results for a version of the model with nominal wage
%stickiness and variable capacity utilization, set STICKY=1 (remember that
%in the the crisis experiment (Figure 7), I assume stikcy wages and
%variable capacity utilization anyway. To generate results for
%alternative values of the investment adjustment cost or the bankruptcy
%cost, vary eta_i and mu below.
%To generate the moments stochastic driving processes as in 
%Smets and Wouters (2007) (table 8-10), set SW=1;

%This file calls Fullmod_dep_crisis.mod, BGG_dep.mod, GK_dep.mod,
%Nofrictions_dep.mod, Figures_paper.m and Figure_scatter.m.



%Choose calibration. Baseline: SW=0 and STICKY=0.
clear all;
SW=0;
STICKY=0;
if SW==1
        STICKY=1;
end


%% Crisis shocks%%
%first entry in Crisis: e_z, second entry: e_N
Crisis    =   [0.0476*100 0.05/11*100];
duration_e_z=   15;
duration_e_N=   11;
save tempCr Crisis duration_e_z duration_e_N;


%% Model parameters%%

%Households
beta    =    0.9958;
hh       =  0.6;
varphi  =   STICKY*1.5+ (1-STICKY)*0.25;
s_nr    =   0; %Share of liquidity constrained households

%Wage Setting;
epsilonw=   STICKY*4+ (1-STICKY)*100;
xiw     =   STICKY*0.75+ (1-STICKY)*0.00001;
gamma_W =   0;

%Entrepreneurs
alpha   =   0.33;
delta   =   0.025;
sigma_ss=   0.35;
pin     =  3.141592653589793;
Theta   =   1;
gamma   =   0.975;
c_U     =   STICKY*1.5+ (1-STICKY)*100;
psi_k   =   1;
psi_l   =   1;
mu      =   0.298122852301025;

%bankers
theta_ss =   0.9915;
Thetab  =   1;

%Capital good's producers
eta_i   =   4;

%Retailers
xi      =   0.67;
gamma_P =   0;
epsilon =   6;
S_ss    =   1;

%Monetary Policy
kappa_pi=   1.5;
kappa_dpi=  0.18;
kappa_X =   0;
kappa_Y =   0.5/4;
rho_i   =   0.8;
kappa_GDP=  0;

%%%%%%%%%%%%%%%%%
%shock process%%
%%%%%%%%%%%%%%%%%

rho_a   =   SW*0.95+ (1-SW)*0.9;
rho_g   =   0.97;
rho_a_I =   0.71;
rho_e_lambda = 0.5;
sd_a    =   SW*0.45/(1-alpha)+ (1-SW)*0.012*100;
sd_i    =   SW*0.24+ (1-SW)*0.0016*100;
sd_g    =   0.52*SW;
sd_I_a_I=   0.45*SW;
sd_e_N  =   0.01*100;
sd_Rb   =   0;
sd_e_z  =   0.05*100;
sd_e_lambda= 0;
sd_e_c   =   0;
rho_e_c  =   SW*0.18;

%extra shocks
sd_e_p =    0.14*SW;
rho_e_p=    0.9;
mu_e_p =    0.74;

sd_e_w =    0.24*SW;
rho_e_w=    0.97;
mu_e_w =    0.88;

sd_e_sigma= 0;
rho_sigma = 0.97;

%calibrated Steady State Values
infl_ss =   1.0223^(1/4);
govshare=   0.2;
s2_ss   =   1.002^(1/4);
l_ss    =   0.33333333;            
phi_b_ss=   1/0.125;
brate   =   0.0075;       

for modtype=1:6 %modtype 1: Executes full model, 2: 
                %BGG model, 3: nofriction model, 4: GK model, 5: Full and GK model to create scatter plot, 6: Crisis experiment, i.e. Full, BGG and GK model
     if modtype<6;
           for MOM=0:1; %MOM=0: Does not calculate second moments, MOM=1: calculates moments
                save temp_simtype modtype MOM
                if MOM ==1; %Make sure that financial shocks are zero when 
                            %calculating second moments, and positive when 
                            %calculating IRFs. 
                        sd_e_N  =   0;
                        sd_e_z  =   0;
                else 
                        sd_e_N  =   0.01*100;
                        sd_e_z  =   0.05*100;
                end
                %Save parameter vector
                p_Full= [beta hh varphi s_nr epsilonw xiw gamma_W alpha delta sigma_ss pin Theta gamma c_U psi_k psi_l mu theta_ss Thetab eta_i xi gamma_P epsilon S_ss kappa_pi kappa_dpi kappa_X kappa_Y rho_i kappa_GDP rho_a rho_g rho_a_I rho_e_lambda rho_e_c sd_a sd_i sd_g sd_I_a_I sd_e_N sd_e_z sd_e_lambda sd_e_c infl_ss govshare s2_ss l_ss phi_b_ss brate sd_e_p rho_e_p mu_e_p sd_e_w rho_e_w mu_e_w sd_e_sigma rho_sigma];
                save temp1 p_Full;
                if  modtype==1; %Save IRF under appropriate name if moment
                                %calculation is switched off
                        MOM
                        dynare Fullmod_dep_crisis;
                        if MOM==0;
                            save temp_Full_IRF;
                        end

                elseif  modtype==2;
                        MOM
                        dynare BGG_dep
                        if MOM==0;
                            save temp_BGG_IRF;
                        end                        

                elseif  modtype==3 && MOM==0;
                        MOM
                        dynare nofrictions_dep
                        if MOM==0;
                            save temp_nofr_IRF;
                        end                        
                elseif modtype==4;
                       MOM
                       dynare GK_dep

                        if MOM==0;
                            save temp_GK_IRF;
                        end
                end


                if MOM==1 && (modtype==3)==0 && (modtype==5)==0; %If moments were calculated 
                                              % name them.
                            sd_GDP  =   sqrt(oo_.var(1,1));
                            sd_Cp   =   sqrt(oo_.var(2,2));
                            sd_I    =   sqrt(oo_.var(3,3));
                            sd_dl   =   sqrt(oo_.var(4,4));
                            sd_ds4_APR= sqrt(oo_.var(5,5));
                            sd_phi1 =   sqrt(oo_.var(6,6));
                            sd_N    =   sqrt(oo_.var(7,7));
                            sd_Rat  =   sqrt(oo_.var(8,8));
                            sd_L    =   sqrt(oo_.var(9,9));
                            sd_dNb  =   sqrt(oo_.var(10,10));
                            sd_Profratio=sqrt(oo_.var(11,11));

                            load temp_simtype;
                            if modtype==4; %For GK model name also moments
                                           %for "Fair value" type variables
                                sd_L_Fair_GK =  sqrt(oo_.var(12,12));
                                sd_Rat_Fair_GK= sqrt(oo_.var(13,13));
                                sd_N_b_Fair_GK =   sqrt(oo_.var(14,14));
                            end

                            %Calculate relative standard deviations
                            Rel_Cp  =   sd_Cp/sd_GDP;
                            Rel_I   =   sd_I/sd_GDP;
                            Rel_dl  =   sd_dl/sd_GDP;
                            Rel_ds4_APR=sd_ds4_APR/sd_GDP;
                            Rel_phi1=   sd_phi1/sd_GDP;
                            Rel_N   =   sd_N/sd_GDP;
                            Rel_Rat =   sd_Rat/sd_GDP;
                            Rel_dL  =   sd_L/sd_GDP;
                            Rel_dNb =   sd_dNb/sd_GDP;
                            Rel_Profratio=sd_Profratio/sd_GDP;

                            %Store relative standard deviations
                            Rel=[Rel_Cp; Rel_I; Rel_dl; Rel_ds4_APR; Rel_phi1; Rel_N; Rel_Rat; Rel_dL; Rel_dNb; Rel_Profratio];
                            
                            %For GK model calculate relative standard 
                            %devations for "Fair value" type variables as 
                            %well.
                            if modtype==4;
                                Rel_L_Fair_GK=sd_L_Fair_GK/sd_GDP;
                                Rel_Rat_Fair_GK=sd_Rat_Fair_GK/sd_GDP;
                                Rel_N_b_Fair_GK=sd_N_b_Fair_GK/sd_GDP;
                                Rel=[Rel_Cp; Rel_I; Rel_dl; Rel_ds4_APR; Rel_phi1; Rel_N; Rel_Rat; Rel_dL; Rel_dNb; Rel_Profratio; Rel_L_Fair_GK; Rel_Rat_Fair_GK; Rel_N_b_Fair_GK];
                            end
                            %Determine size of vectors for Correlation
                            %coefficents (bigger for GK model since Fair
                            %value type variables are included.
                            n=size(Rel);

                            COR=[];
                            ACOR=[];
                            %Calculate crosscorrelations and
                            %Autocorrelations
                            for k=1:n(1)+1;
                                COR=[ COR; oo_.var(k,1)/sqrt(oo_.var(1,1)*oo_.var(k,k))];
                                ACOR=[ ACOR; oo_.autocorr{1,1}(k,k)];
                            end
                            Rel
                            COR
                            ACOR



                    load temp_simtype;
                    %Save second moments in appropriately named .mat file
                    if modtype==1
                        Rel_Full=Rel;
                        Cor_Full=COR;
                        ACOR_Full=ACOR;
                        save temp_Mo_Full Rel_Full Cor_Full ACOR_Full;

                    elseif modtype==2
                        Rel_BGG=Rel;
                        Cor_BGG=COR;
                        ACOR_BGG=ACOR;
                        save temp_Mo_BGG Rel_BGG Cor_BGG ACOR_BGG;

                    elseif modtype==4
                        Rel_GK=Rel;
                        Cor_GK=COR;
                        ACOR_GK=ACOR;
                        save temp_Mo_GK Rel_GK Cor_GK ACOR_GK;
                    end

                end
                
           end
           
           %Produce figures after GK model has been executed, Scatterplot
           %after that.
           if modtype==4;          
                  Figures_paper
           elseif modtype==5;
                  Figure_scatter;
           end
     end
     
     %Crisis experiment
     if modtype==6;
           save temp_simtype modtype;
           %Number of periods in crisis experiment. If this is changed, it
           %has to be changed in the .mod files as well. 
           periods=40;
           
           %Matrices where responses to bank and entrepreneurial net worth
           %are stored temporarily and finally. 
           Matr_Crisis_e_z=zeros(periods,19);
           Matr_Crisis_e_N=zeros(periods,19);
           Matr_Crisis_Full_e_z=zeros(periods,19);
           Matr_Crisis_GK_e_z=zeros(periods,19);
           Matr_Crisis_Full_e_N=zeros(periods,19);
           Matr_Crisis_BGG_e_N=zeros(periods,19);

           save temp3a Matr_Crisis_e_z;
           save temp3b Matr_Crisis_e_N;
           save temp_Matr_ez Matr_Crisis_Full_e_z Matr_Crisis_GK_e_z;
           save temp_Matr_e_N Matr_Crisis_Full_e_N Matr_Crisis_BGG_e_N;

                for M=1:3; %Loop going through the three models (1: Full 
                           %model, 2: BGG, 3: GK model)
                    if M==1 || M==3; %Loop for banak net worth shocks, only 
                                     %applicable to full and GK model
                        M
                        load tempCr;
                            if Crisis(1)>0;
                                        %Set all unecessary shocks equal to
                                        %zero, set sd_e_z equal to one period
                                        %net worth shock, switch on variable capacity
                                        %utilization and sticky wages. 
                                        sd_a    =   0;
                                        sd_i    =   0;
                                        sd_g    =   0;
                                        sd_I_a_I=   0;
                                        sd_e_p  =   0;
                                        sd_e_w  =   0;
                                        sd_e_N  =   0;
                                        sd_Rb   =   0;
                                        sd_e_z  =   Crisis(1);
                                        sd_e_c  =   0;
                                        sd_e_lambda=0;
                                        rho_e_lambda=0;
                                        rho_e_c =   0;
                                        c_U     =   1.5;
                                        xiw     =   0.75;
                                        varphi  =   1.5;
                                        epsilonw=   4;

                                        %Save parameter vector
                                        p_Full= [beta hh varphi s_nr epsilonw xiw gamma_W alpha delta sigma_ss pin Theta gamma c_U psi_k psi_l mu theta_ss Thetab eta_i xi gamma_P epsilon S_ss kappa_pi kappa_dpi kappa_X kappa_Y rho_i kappa_GDP rho_a rho_g rho_a_I rho_e_lambda rho_e_c sd_a sd_i sd_g sd_I_a_I sd_e_N sd_e_z sd_e_lambda sd_e_c infl_ss govshare s2_ss l_ss phi_b_ss brate sd_e_p rho_e_p mu_e_p sd_e_w rho_e_w mu_e_w sd_e_sigma rho_sigma];
                                        save temp1 p_Full;
                                        counter = 0;
                                        save temp5 counter;
                                        load temp3a Matr_Crisis_e_z;
                                        Matr_Crisis_e_z=zeros(periods,19);
                                        save temp3a Matr_Crisis_e_z;
                                        save temp4 periods M;
                                            if M==1; 
                                                dynare Fullmod_dep_crisis
                                                Matr_e_z=[GDP_Full_e_z R_4_Full_e_z C_p_Full_e_z I_Full_e_z l_Full_e_z infl_4_Full_e_z Rk_4_Full_e_z s3_4_Full_e_z Q_Full_e_z N_Full_e_z N_b_Full_e_z phi_b_Full_e_z L_Full_e_z Rat_Full_e_z s2_4_Full_e_z L_e_Full_e_z s4_4_Full_e_z Rat_Full_e_z N_b_Full_e_z];
                                            elseif M==3;
                                                dynare GK_dep
                                                Matr_e_z=[GDP_GK_e_z R_4_GK_e_z C_p_GK_e_z I_GK_e_z l_GK_e_z infl_4_GK_e_z Rk_4_GK_e_z s3_4_GK_e_z Q_GK_e_z N_GK_e_z N_b_GK_e_z phi_b_GK_e_z L_GK_e_z Rat_GK_e_z s2_4_GK_e_z L_e_GK_e_z s3_4_GK_e_z N_b_Fair_GK_e_z Rat_Fair_GK_e_z];
                                            end
                                        load tempCr;
                                        for k=1:duration_e_z;

                                            Matr_step1=Matr_e_z;
                                            load temp4;
                                            Matr_step2=[zeros(k-1,19); Matr_step1(1:(periods-k+1),:)];
                                            load temp3a Matr_Crisis_e_z;
                                            Matr_Crisis_e_z=Matr_Crisis_e_z+Matr_step2;
                                            save temp3a Matr_Crisis_e_z;
                                            load temp5;
                                            counter=1+counter;
                                            save temp5 counter;
                                        end

                                        load temp_Matr_ez;
                                        load temp3a

                                        if M==1
                                            Matr_Crisis_Full_e_z=Matr_Crisis_e_z;

                                        elseif M==3
                                            Matr_Crisis_GK_e_z=Matr_Crisis_e_z;
                                        end

                                        save temp_Matr_ez Matr_Crisis_Full_e_z Matr_Crisis_GK_e_z;
                            end
                    end
                    load tempCr;
                    if M==1 || M==2;
                            M
                            if Crisis(2)>0;
                                         sd_e_N  =   Crisis(2);
                                         sd_e_z  =   0;
                                         p_Full= [beta hh varphi s_nr epsilonw xiw gamma_W alpha delta sigma_ss pin Theta gamma c_U psi_k psi_l mu theta_ss Thetab eta_i xi gamma_P epsilon S_ss kappa_pi kappa_dpi kappa_X kappa_Y rho_i kappa_GDP rho_a rho_g rho_a_I rho_e_lambda rho_e_c sd_a sd_i sd_g sd_I_a_I sd_e_N sd_e_z sd_e_lambda sd_e_c infl_ss govshare s2_ss l_ss phi_b_ss brate sd_e_p rho_e_p mu_e_p sd_e_w rho_e_w mu_e_w sd_e_sigma rho_sigma];
                                         save temp1 p_Full;
                                         counter = 0;
                                         save temp5 counter;
                                         load temp3b Matr_Crisis_e_N;
                                         Matr_Crisis_e_N=zeros(periods,19);
                                         save temp3b Matr_Crisis_e_N;
                                         save temp4 periods M;
                                         if M==1; 
                                                dynare Fullmod_dep_crisis
                                                Matr_e_N=[GDP_Full_e_N R_4_Full_e_N C_p_Full_e_N I_Full_e_N l_Full_e_N infl_4_Full_e_N Rk_4_Full_e_N s3_4_Full_e_N Q_Full_e_N N_Full_e_N N_b_Full_e_N phi_b_Full_e_N L_Full_e_N Rat_Full_e_N s2_4_Full_e_N L_e_Full_e_N s4_4_Full_e_N Rat_Full_e_N N_b_Full_e_N];
                                            elseif M==2;
                                                dynare BGG_dep
                                                Matr_e_N=[GDP_BGG_e_N R_4_BGG_e_N C_p_BGG_e_N I_BGG_e_N l_BGG_e_N infl_4_BGG_e_N Rk_4_BGG_e_N s3_4_BGG_e_N Q_BGG_e_N N_BGG_e_N N_b_BGG_e_N phi_b_BGG_e_N L_BGG_e_N Rat_BGG_e_N s2_4_BGG_e_N L_e_BGG_e_N s4_4_BGG_e_N Rat_BGG_e_N N_b_BGG_e_N];
                                         end
                                         
                                        load tempCr;
                                        for k=1:duration_e_N;
                                            
                                            Matr_step1=Matr_e_N;
                                            load temp4;
                                            Matr_step2=[zeros(k-1,19); Matr_step1(1:(periods-k+1),:)];
                                            load temp3b Matr_Crisis_e_N;
                                            Matr_Crisis_e_N=Matr_Crisis_e_N+Matr_step2;
                                            save temp3b Matr_Crisis_e_N;
                                            load temp5;
                                            counter=1+counter;
                                            save temp5 counter;
                                        end

                                        load temp_Matr_e_N;

                                        if M==1
                                            Matr_Crisis_Full_e_N=Matr_Crisis_e_N;

                                        elseif M==2
                                            Matr_Crisis_BGG_e_N=Matr_Crisis_e_N;
                                        end

                                        save temp_Matr_e_N Matr_Crisis_Full_e_N Matr_Crisis_BGG_e_N;




                            end
                    end
                end;

            load temp_Matr_ez;
            load temp_Matr_e_N;
            Matr_Crisis_Full=Matr_Crisis_Full_e_z+Matr_Crisis_Full_e_N;
            Matr_Crisis_GK=Matr_Crisis_GK_e_z;
            Matr_Crisis_BGG=Matr_Crisis_BGG_e_N;


        GDP_Full_Crisis  =Matr_Crisis_Full(:,1);
        R_4_Full_Crisis  =Matr_Crisis_Full(:,2);
        C_p_Full_Crisis  =Matr_Crisis_Full(:,3);
        I_Full_Crisis    =Matr_Crisis_Full(:,4);
        l_Full_Crisis    =Matr_Crisis_Full(:,5);
        infl_4_Full_Crisis=Matr_Crisis_Full(:,6);
        Rk_4_Full_Crisis =Matr_Crisis_Full(:,7);
        s3_4_Full_Crisis =Matr_Crisis_Full(:,8);
        Q_Full_Crisis    =Matr_Crisis_Full(:,9);
        N_Full_Crisis    =Matr_Crisis_Full(:,10);
        N_b_Full_Crisis  =Matr_Crisis_Full(:,11);
        phi_b_Full_Crisis=Matr_Crisis_Full(:,12);
        L_Full_Crisis    =Matr_Crisis_Full(:,13);
        Rat_Full_Crisis  =Matr_Crisis_Full(:,14);
        s2_4_Full_Crisis =Matr_Crisis_Full(:,15);
        L_e_Full_Crisis  =Matr_Crisis_Full(:,16);
        s4_4_Full_Crisis =Matr_Crisis_Full(:,17);

        GDP_BGG_Crisis  =Matr_Crisis_BGG(:,1);
        R_4_BGG_Crisis  =Matr_Crisis_BGG(:,2);
        C_p_BGG_Crisis  =Matr_Crisis_BGG(:,3);
        I_BGG_Crisis    =Matr_Crisis_BGG(:,4);
        l_BGG_Crisis    =Matr_Crisis_BGG(:,5);
        infl_4_BGG_Crisis=Matr_Crisis_BGG(:,6);
        Rk_4_BGG_Crisis =Matr_Crisis_BGG(:,7);
        s3_4_BGG_Crisis =Matr_Crisis_BGG(:,8);
        Q_BGG_Crisis    =Matr_Crisis_BGG(:,9);
        N_BGG_Crisis    =Matr_Crisis_BGG(:,10);
        N_b_BGG_Crisis  =Matr_Crisis_BGG(:,11);
        phi_b_BGG_Crisis=Matr_Crisis_BGG(:,12);
        L_BGG_Crisis    =Matr_Crisis_BGG(:,13);
        Rat_BGG_Crisis  =Matr_Crisis_BGG(:,14);
        s2_4_BGG_Crisis =Matr_Crisis_BGG(:,15);
        L_e_BGG_Crisis  =Matr_Crisis_BGG(:,16);
        s4_4_BGG_Crisis =Matr_Crisis_BGG(:,17);

        GDP_GK_Crisis  =Matr_Crisis_GK(:,1);
        R_4_GK_Crisis  =Matr_Crisis_GK(:,2);
        C_p_GK_Crisis  =Matr_Crisis_GK(:,3);
        I_GK_Crisis    =Matr_Crisis_GK(:,4);
        l_GK_Crisis    =Matr_Crisis_GK(:,5);
        infl_4_GK_Crisis=Matr_Crisis_GK(:,6);
        Rk_4_GK_Crisis =Matr_Crisis_GK(:,7);
        s3_4_GK_Crisis =Matr_Crisis_GK(:,8);
        Q_GK_Crisis    =Matr_Crisis_GK(:,9);
        N_GK_Crisis    =Matr_Crisis_GK(:,10);
        N_b_GK_Crisis  =Matr_Crisis_GK(:,11);
        phi_b_GK_Crisis=Matr_Crisis_GK(:,12);
        L_GK_Crisis    =Matr_Crisis_GK(:,13);
        Rat_GK_Crisis  =Matr_Crisis_GK(:,14);
        s2_4_GK_Crisis =Matr_Crisis_GK(:,15);
        L_e_GK_Crisis  =Matr_Crisis_GK(:,16);
        s3_4_GK_Crisis =Matr_Crisis_GK(:,17);
        N_b_Fair_GK_Crisis=Matr_Crisis_GK(:,18);
        Rat_Fair_GK_Crisis=Matr_Crisis_GK(:,19);

        save temp_Crisis;
     end
end;

Figure_crisis;
    